/** 
 * Name analysis for expressions
 */

package expression;

import java.io.*;
import expression.Tree.*;

/** name analyser visitor
 */
public class Analyzer implements Tree.Visitor, Tokens {
    
    /** current scope
     */
    Scope scope;        

    /** construct a new semantic analyzer
     */
    public Analyzer() {
        this.scope = null;
    }
    
    /** the main name analysis method
     */
    public static void analyzeTree(Tree tree) {
        tree.apply(new Analyzer());
    }
    
    /** analysis method for recursion
     *  we use the visitor reuse version
     */
    protected void analyze(Tree tree) {
        tree.apply(this);
    }
    
    /** open a new nested scope
     */
    protected void openScope() {
        scope = new Scope(scope);
    }
    
    /** close a nested scope
     */
    protected void closeScope() {
        scope = scope.outer;
    }
    
    /** the visitor methods
     */            
    public void caseProgram(Program tree) {
	openScope();
	for (int i = 0; i < tree.defs.length; i ++) {
	    analyze(tree.defs[i]);
	}
	closeScope();
    }
    
    public void caseDef(Def tree) {
	analyze (tree.expr);
	// fill: enter definition
    }
    
    
    public void caseIntLiteral(IntLiteral tree) {
    }
        
    public void caseIdent(Ident tree) {
	// fill: retrieve defintion 
    }
        
    public void caseBinOp(BinOp tree) {
	analyze(tree.left);
	analyze(tree.right);
    }
    
    
    public void caseBad(Bad tree) {
    }

}













