/** 
 * A paser for expressions
 */

package expression;

import java.io.*;
import java.util.*;
import expression.Tree.*;

/** the parser class
 */
class Parser extends Scanner {

    /** the last scanned token
     */
    int token;

    /** parse a list of definitions
     */
    public Tree defs() throws IOException {
	// use l to collect all definitions
	List l = new ArrayList();
	while (token == IDENT) {
	    String name = chars;
	    int p = pos;
	    token = nextToken();
	    if (token == EQUALS) {
		token = nextToken();
	    } else {
		Report.error(pos, "\'=\' expected.");
	    }
	    Tree expr = expression();
	    if (token == SEMI) {
		token = nextToken();
	    } else {
		Report.error(pos, "\';\' expected.");
	    }
	    l.add (new Def(p, name, expr));
	}
	// put all definitions from l into a new array
	return new Program(pos, (Tree[]) l.toArray(new Tree[l.size()]));
    }

    /** parse an expression
     */
    public Tree expression() throws IOException {
	Tree t = term();
	while (token == MINUS || token == PLUS) {
	    char op;
	    if (token == MINUS) {
		op = '-';
	    } else {
		op = '+'; 
	    }
	    int p = pos;
	    token = nextToken();
	    t = new BinOp(p, op, t, term());
	}
	return t;
    }

    /** parse a term
     */
    public Tree term() throws IOException {
	// fill: tree generation code
	Tree t = factor();
	while (token == TIMES || token == DIV) {
	    token = nextToken();
	    factor();
	}
	return t;
    }

    /** parse a factor
     */
    public Tree factor() throws IOException {
	// fill: add cases for IDENT and "(" Expr ")"
	switch(token) {
	case INTLIT: {
	    int i = Integer.parseInt(chars);
	    int p = pos;
	    token = nextToken();
	    return new IntLiteral(p, i);
	}
	default: {
	    Report.error(pos, "\'(\', integer or identifier expected.");
	    return new Bad(pos);
	}
	}
    }

    /** parse the whole program
     */
    public Tree parse() throws IOException {
	Tree t = defs();
	if (token != EOF) {
	    Report.error(pos, "EOF expected.");
	}
	return t;
    }

    /** construct a new parser reading from in
     */
    public Parser(InputStream in) throws IOException {
        super(in);
	// read first token
        token = nextToken();
    }

    /** main function for testing parser
     *  (for debugging)
     */
    static public void main(String[] args) {
        try {
            Parser p = new Parser(System.in);
	    p.parse();
	    if (Report.errCount == 0) {
                System.out.println("Parsing successful.");
            } else {
                System.out.println(Report.errCount + " errors during parsing.");
            }
        } catch (IOException e) {
            System.out.println(e.toString());
        }
    }
}           
